package com.appbackend.aleph.controller;

import com.appbackend.aleph.exception.AlephException;
import com.appbackend.aleph.model.*;
import com.appbackend.aleph.utils.DateUtils;
import com.appbackend.aleph.utils.AlephServiceProcessor;
import com.appbackend.aleph.utils.XmlUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import java.io.IOException;
import java.util.*;

@Controller
public class AlephController {

    @Value("${aleph.and.separator}")
    private String AND_SYMBOL;

    @Autowired
    private AlephServiceProcessor alephServiceProcessor;

    private static Map<String, String> MAP_WITH_STATUSES = createMapWithStatus();
    private static Map<String, String> MAP_WITH_COLLECTIONS = createMapWithCollection();

    @RequestMapping(value = "/aleph/userInfo", method = RequestMethod.POST)
    @ResponseBody
    public AlephUser userInfo(@RequestBody Credentials credentials) {
        String request = "op=bor-info" + AND_SYMBOL + "library=CZU50" + AND_SYMBOL + "bor_id=" + credentials.getUsername() + AND_SYMBOL + "verification=" + credentials.getPassword();
        try {

            Document document = alephServiceProcessor.processAlephRequest(request);

            XPathFactory xpathFactory = XPathFactory.newInstance();
            XPath xpath = xpathFactory.newXPath();

            NodeList nodeList = document.getElementsByTagName("item-h");
            List<ReservationBook> reservations = new ArrayList<>();
            for (Node item : XmlUtils.nodeListToList(nodeList)) {
                reservations.add(new ReservationBook(
                        xpath.evaluate("z37/z37-doc-number", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-title", item, XPathConstants.STRING).toString().split("/")[0],
                        xpath.evaluate("z13/z13-author", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-year", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-isbn-issn", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z30/z30-call-no", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z30/z30-pages", item, XPathConstants.STRING).toString().replaceAll("\\D+", ""),
                        null,
                        xpath.evaluate("z37/z37-item-sequence", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z37/z37-sequence", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z37/z37-status", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z37/z37-end-request-date", item, XPathConstants.STRING).toString())
                );
            }

            List<CatalogBook> history = new ArrayList<>();
            nodeList = document.getElementsByTagName("fine");
            for (Node item : XmlUtils.nodeListToList(nodeList)) {
                history.add(new CatalogBook(
                        xpath.evaluate("z36/z36-doc-number", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-title", item, XPathConstants.STRING).toString().split("/")[0],
                        xpath.evaluate("z13/z13-author", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-year", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-isbn-issn", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z30/z30-call-no", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z30/z30-pages", item, XPathConstants.STRING).toString().replaceAll("\\D+", ""),
                        null
                ));
            }

            List<LoanBook> loans = new ArrayList<>();
            nodeList = document.getElementsByTagName("item-l");
            for (Node item : XmlUtils.nodeListToList(nodeList)) {
                String currentFine = xpath.evaluate("current-fine", item, XPathConstants.STRING).toString();
                loans.add(new LoanBook(
                        xpath.evaluate("z36/z36-doc-number", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-title", item, XPathConstants.STRING).toString().split("/")[0],
                        xpath.evaluate("z13/z13-author", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-year", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z13/z13-isbn-issn", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z30/z30-call-no", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("z30/z30-pages", item, XPathConstants.STRING).toString().replaceAll("\\D+", ""),
                        null,
                        xpath.evaluate("z30/z30-barcode", item, XPathConstants.STRING).toString(),
                        DateUtils.parseDateToString(DateUtils.parseStringToDate("dd/MM/yyyy", xpath.evaluate("z36/z36-due-date", item, XPathConstants.STRING).toString())),
                        (currentFine != null && currentFine.isEmpty()) ? Double.parseDouble(currentFine) : 0.0)
                );
            }

            nodeList = document.getElementsByTagName("z303");
            Node userTag = XmlUtils.nodeListToList(nodeList).get(0);
            return new AlephUser(
                    credentials.getUsername(),
                    xpath.evaluate("z303-id", userTag, XPathConstants.STRING).toString(),
                    xpath.evaluate("z303-name", userTag, XPathConstants.STRING).toString(),
                    loans,
                    reservations,
                    history
            );
        } catch (IOException | ParserConfigurationException | SAXException | XPathExpressionException e) {
            e.printStackTrace();
            throw new AlephException("Error during parsing response from Aleph op=bor-info");
        }
    }

    @RequestMapping(value = "/aleph/find", method = RequestMethod.GET)
    @ResponseBody
    public FindResult find(@RequestParam("query") String query,
                           @RequestParam("filter") String filter) {
        switch (filter) {
            case "title":
                filter = "WTI";
                break;
            case "author":
                filter = "WAU";
                break;
            case "signature":
                filter = "SIG";
                break;
        }
        try {
            String encapsulatedQuery = query.replace(" ","%26");
            String request = "op=find" + AND_SYMBOL + "base=CZU01" + AND_SYMBOL + "request=" + filter + "=(" + encapsulatedQuery + ")";

            Document document = alephServiceProcessor.processAlephRequest(request);
//            if (document.getElementsByTagName("error").item(0) != null && "empty set".equals(document.getElementsByTagName("error").item(0).getTextContent()))
//                throw new AlephException("No result for query: " + query);

            return new FindResult(document.getElementsByTagName("set_number").item(0).getTextContent(),
                    Integer.parseInt(document.getElementsByTagName("no_entries").item(0).getTextContent()));
        } catch (ParserConfigurationException | SAXException | IOException e) {
            e.printStackTrace();
            throw new AlephException("Error during parsing response from Aleph op=find");
        }
    }

    @RequestMapping(value = "/aleph/present", method = RequestMethod.GET)
    @ResponseBody
    public List<CatalogBook> present(@RequestParam(value = "setNumber") String setNumber,
                                     @RequestParam(value = "from") int from,
                                     @RequestParam(value = "loadBy") int loadBy) {

        String request = "op=present" + AND_SYMBOL + "set_entry=" + from + "-" + (from + loadBy) + AND_SYMBOL + "set_number=" + setNumber;

        try {
            Document document = alephServiceProcessor.processAlephRequest(request);
            XPathFactory xpathFactory = XPathFactory.newInstance();
            XPath xpath = xpathFactory.newXPath();

            NodeList nodeList = document.getElementsByTagName("record");
            List<CatalogBook> catalogBooks = new ArrayList<>();

            for (Node item : XmlUtils.nodeListToList(nodeList)) {
                String year = xpath.evaluate("metadata/oai_marc/varfield[@id='260']/subfield[@label='c']", item, XPathConstants.STRING).toString();
                if (year.isEmpty())
                    year = xpath.evaluate("metadata/oai_marc/varfield[@id='264']/subfield[@label='c']", item, XPathConstants.STRING).toString();

                year = year.replaceAll("[^0-9]", "");

                String title = xpath.evaluate("metadata/oai_marc/varfield[@id='245']/subfield[@label='a']", item, XPathConstants.STRING).toString();
                String subtitle = xpath.evaluate("metadata/oai_marc/varfield[@id='245']/subfield[@label='b']", item, XPathConstants.STRING).toString();
//                title = title.concat(" " + subtitle);
                title = title.replaceAll(":$|/$", "");

                catalogBooks.add(new CatalogBook(
                                xpath.evaluate("doc_number", item, XPathConstants.STRING).toString().isEmpty() ? null : xpath.evaluate("doc_number", item, XPathConstants.STRING).toString().replaceFirst("^0+(?!$)", ""),
                                title,
                                xpath.evaluate("metadata/oai_marc/varfield[@id='245']/subfield[@label='c']", item, XPathConstants.STRING).toString().replaceAll("\\.", ""),
                                year,
                                xpath.evaluate("metadata/oai_marc/varfield[@id='020']/subfield[@label='a']", item, XPathConstants.STRING).toString(),
                                null,
                                xpath.evaluate("metadata/oai_marc/varfield[@id='300']/subfield[@label='a']", item, XPathConstants.STRING).toString().replaceAll("\\D+", ""),
                                xpath.evaluate("metadata/oai_marc/varfield[@id='250']/subfield[@label='a']", item, XPathConstants.STRING).toString())
//                    null
                );

            }
            return catalogBooks;
        } catch (IOException | ParserConfigurationException | SAXException | XPathExpressionException e) {
            e.printStackTrace();
            throw new AlephException("Error during parsing response from Aleph op=present");
        }
    }

//    @RequestMapping(value = "/aleph/findAndPresent", method = RequestMethod.GET)
//    public List<CatalogBook> findAndPresent(@RequestParam(value = "query") String query,
//                                            @RequestParam(value = "filter") String filter,
//                                            @RequestParam(value = "from") int from,
//                                            @RequestParam(value = "loadBy") int loadBy) {
//        FindResult result = findBook(query, filter);
//        return present(result.getSetNumber(), from, loadBy);
//    }

    @RequestMapping(value = "/aleph/itemData", method = RequestMethod.GET)
    @ResponseBody
    public List<Item> itemData(@RequestParam(value = "docNumber") String docNumber) {
        String request = "op=item-data" + AND_SYMBOL + "doc_number=" + docNumber  + AND_SYMBOL +  "base=CZU01";
        Document document = null;

        try {
            document = alephServiceProcessor.processAlephRequest(request);

            XPathFactory xpathFactory = XPathFactory.newInstance();
            XPath xpath = xpathFactory.newXPath();

            NodeList nodeList = document.getElementsByTagName("item");
            List<Item> items = new ArrayList<>();

            for (Node item : XmlUtils.nodeListToList(nodeList)) {
                String itemStatus = xpath.evaluate("item-status", item, XPathConstants.STRING).toString();
                String formatedDueDate = DateUtils.parseDateToString(DateUtils.parseStringToDate("yyyyMMdd", xpath.evaluate("loan-due-date", item, XPathConstants.STRING).toString()));
                items.add(new Item(docNumber,
                        xpath.evaluate("rec-key", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("barcode", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("call-no-1", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("call-no-2", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("sub-library", item, XPathConstants.STRING).toString(),
                        MAP_WITH_COLLECTIONS.get(xpath.evaluate("collection", item, XPathConstants.STRING).toString()),
                        MAP_WITH_STATUSES.get(itemStatus),
                        xpath.evaluate("note", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("library", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("on-hold", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("requested", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("expected", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("loan-status", item, XPathConstants.STRING).toString(),
                        xpath.evaluate("loan-in-transit", item, XPathConstants.STRING).toString(),
                        formatedDueDate,
                        createState(itemStatus, formatedDueDate)
                ));
            }
            return items;
        } catch (IOException | ParserConfigurationException | SAXException | XPathExpressionException e) {
            e.printStackTrace();
            throw new AlephException("Error during parsing response from Aleph op=itemData");
        }

    }

    @RequestMapping(value = "/aleph/holdRequest", method = RequestMethod.GET)
    @ResponseBody
    public void holdRequest(@RequestParam(value = "docNumber") String docNumber,
                            @RequestParam(value = "itemBarCode") String itemBarCode,
                            @RequestParam(value = "borId") String borId) {

        String request = "op=hold-req" + AND_SYMBOL + "item_barcode=" + itemBarCode + AND_SYMBOL + "bor_id=" + borId + AND_SYMBOL + "library=CZU50";
        try {
            alephServiceProcessor.processAlephRequest(request);
        } catch (IOException | ParserConfigurationException | SAXException e) {
            e.printStackTrace();
            throw new AlephException("Error during parsing response from Aleph op=hold-req");
        }
    }

    @RequestMapping(value = "/aleph/cancelHoldRequest", method = RequestMethod.GET)
    @ResponseBody
    public void cancelHoldRequest(@RequestParam(value = "docNumber") String docNumber,
                               @RequestParam(value = "itemSequence") String itemSequence,
                               @RequestParam(value = "sequence") String sequence) {
        String request = "op=hold-req-cancel" + AND_SYMBOL + "doc_number=" + docNumber + AND_SYMBOL + "item_sequence=" + itemSequence + AND_SYMBOL + "sequence=" + sequence + AND_SYMBOL + "library=CZU50";
        try {
            alephServiceProcessor.processAlephRequest(request);
        } catch (IOException | ParserConfigurationException | SAXException e) {
            e.printStackTrace();
            throw new AlephException("Error during parsing response from Aleph op=hold-req-cancel");
        }
    }

    @RequestMapping(value = "/aleph/renewLoan", method = RequestMethod.GET)
    @ResponseBody
    public Date renewLoan(@RequestParam(value = "itemBarCode") String itemBarCode,
                          @RequestParam(value = "borId") String borId) {

        String request = "op=renew" + AND_SYMBOL + "item_barcode=" + itemBarCode +
                AND_SYMBOL + "bor_id=" + borId + AND_SYMBOL + "library=CZU50";

        try {
            Document document = alephServiceProcessor.processAlephRequest(request);

            XPathFactory xpathFactory = XPathFactory.newInstance();
            XPath xpath = xpathFactory.newXPath();

            NodeList nodeList = document.getElementsByTagName("renew");
            Node renewTag = XmlUtils.nodeListToList(nodeList).get(0);
            return DateUtils.parseStringToDate("yyyyMMdd", xpath.evaluate("due-date", renewTag, XPathConstants.STRING).toString());

        } catch (IOException | ParserConfigurationException | XPathExpressionException | SAXException e) {
            e.printStackTrace();
            throw new AlephException("Error during parsing response from Aleph op=hold-req-cancel");
        }

    }

    @RequestMapping(value = "/aleph/feed", method = RequestMethod.GET)
    @ResponseBody
    public List<FeedCard> feed() {

        String mainUrl = "https://lib.czu.cz";
        String libUrl = mainUrl + "/cs/r-8995-aktuality-sic";

        try {
            List<FeedCard> feedList = new ArrayList<>();
            org.jsoup.nodes.Document doc = Jsoup.connect(libUrl).get();

            Elements cards = doc.select(".article-card");
            for (Element card : cards) {
                FeedCard feedCard = new FeedCard(
                        card.getElementsByClass("card-title").text(),
                        card.getElementsByClass("card-text date text-muted").text(),
                        card.getElementsByClass("card-text article-perex d-none d-lg-block").text(),
                        mainUrl + card.getElementsByClass("card-img-top").attr("src"),
                        mainUrl + card.getElementsByClass("btn btn-default featured-btn").attr("href"));
                feedList.add(feedCard);
            }
            return feedList;

        } catch (IOException e) {
            e.printStackTrace();
            throw new AlephException("Problem with content from lib.czu.cz");
        }

    }

    private static HashMap<String, String> createMapWithStatus() {
        HashMap<String, String> map = new HashMap<>();
        map.put("01", "Prezenční");
        map.put("02", "Katedra (mimo SIC)");
        map.put("04", "Absenční");
        map.put("05", "Skripta");

        //todo tyhle kody hadam, nepovedlo se mi dohledat..
        map.put("06", "Archiv");
        map.put("07", "Zapůjčeno na MVS");
        map.put("08", "Novinka");
        map.put("11", "Online");
        return map;
    }

    private static String createState(String itemStatus, String formatedDueDate) {
        if(formatedDueDate!=null) {
            return "Exemplář je vypůjčený do: " + formatedDueDate;
        } else {
            switch (itemStatus) {
                case "01" : return "K vypůjčení v rámci budovy knihovny";
                case "04" : return "K vypůjčení na 1 měsíc";
                case "08" : return "K vypůjčení na 1 měsíc";
                case "05" : return "K vypůjčení na 2 měsíce";
                case "11" : return "K vypůjčení online";
                default: return "K vypůjčení je nutné kontaktovat výpůjční službu";
            }
        }
    }

    private static HashMap<String, String> createMapWithCollection()  {
        HashMap<String, String> map = new HashMap<>();
        map.put("351", "Centrum provýzkum");
        map.put("20", "Katedra agroekologiea");
        map.put("111", "Katedra agroekologiea");
        map.put("13", "Katedra agroenvironmentální chemie");
        map.put("17", "Katedra aplik.geoinformatiky");
        map.put("66", "Katedra aplikovanéekologie");
        map.put("KAE", "Katedra aplikovanéekologie");
        map.put("16", "Katedra biotechnickýchúprav");
        map.put("23", "Katedra botanikya");
        map.put("23F", "Katedra botanikya");
        map.put("251", "Katedra ekologie");
        map.put("69", "Katedra ekologielesa");
        map.put("894", "Katedra ekonomikya");
        map.put("27E", "Katedra ekonomickýchteorií");
        map.put("52", "Katedra ekonomiky");
        map.put("ELH", "Katedra lesnické");
        map.put("50", "Katedra elektrotechniky");
        map.put("42", "Katedra fyziky");
        map.put("KD", "Katedra genetikya");
        map.put("37", "Katedra genetikya");
        map.put("65", "Katedra geoenvironmentálních věd");
        map.put("60", "Katedra hospodářskéúpravy");
        map.put("27", "Katedra humanitníchvěd");
        map.put("12", "Katedra chemie");
        map.put("893", "Katedra chovuzvířat");
        map.put("51", "Katedra informačního inženýrství");
        map.put("54", "Katedra informačních technologií");
        map.put("451", "Katedra jakostia");
        map.put("KJ", "Katedra jazyků");
        map.put("64", "Katedra kvality zemědělských");
        map.put("LTD", "Katedra lesnických technologií");
        map.put("43", "Katedra matematiky");
        map.put("452", "Katedra materiálůa");
        map.put("44", "Katedra mechanikya");
        map.put("32", "Katedra mikrobiologie, výživy");
        map.put("14", "Katedra mikrobiologie, výživy");
        map.put("68", "Katedra myslivostia");
        map.put("35", "Katedra obecné zootechniky");
        map.put("90", "Katedra obchodua");
        map.put("61", "Katedra ochranylesa");
        map.put("24", "Katedra ochranyrostlin");
        map.put("KP", "Katedra pedagogiky(");
        map.put("15", "Katedra pedologiea");
        map.put("11", "Katedra pedologiea");
        map.put("62", "Katedra pěstování lesů");
        map.put("23P", "Katedra pícninářstvía");
        map.put("57", "Katedra práva");
        map.put("27P", "Katedra psychologie");
        map.put("21", "Katedra rostlinné výroby");
        map.put("33", "Katedra speciální zootechniky");
        map.put("53s", "Katedra statistiky");
        map.put("55", "Katedra systémového inženýrství");
        map.put("46", "Katedra technologickýchzařízení");
        map.put("891", "Katedra tropických plodin");
        map.put("892", "Katedra udržitelných technologií");
        map.put("31", "Katedra veterinárních disciplín");
        map.put("81s", "Katedra vodního hospodář.");
        map.put("67", "Katedra vodních zdrojů");
        map.put("47", "Katedra vozidel");
        map.put("48", "Katedra využití strojů");
        map.put("26", "Katedra zahradní");
        map.put("25", "Katedra zahradnictví");
        map.put("KZD", "Katedra základního zpracování");
        map.put("49", "Katedra zemědělských strojů");
        map.put("23Z", "Katedra zoologie");
        map.put("ZK", "Základní knihovna SIC");
        return map;
    }

}
