import React from 'react';
import {
    ActivityIndicator,
    ImageSourcePropType,
    NativeSyntheticEvent,
    NativeTouchEvent,
    StyleSheet,
    TouchableHighlight,
    View
} from "react-native";
import {Image} from "react-native-elements";
import {RobotoBoldText, RobotoText} from "components/text/StyledText";
import {GLOBAL_STYLES} from "../style/GlobalStyles";

type Props = {
    title: String;
    author: String;
    year: String;
    isbn: String;
    onPress?: (ev: NativeSyntheticEvent<NativeTouchEvent>) => void;
}

type State = {
   imageSource: ImageSourcePropType
}

export default class BookListItem extends React.PureComponent<Props, State> {

    render() {
        return (
            <TouchableHighlight onPress={this.props.onPress}>
                <View style={styles.bookContainer}>
                    <Image style={styles.bookImage}
                           source={{uri : `https://www.obalkyknih.cz/api/cover?isbn=${this.props.isbn}`}}
                           PlaceholderContent={<ActivityIndicator
                               style={GLOBAL_STYLES.indicator}/>
                           }
                           />
                    <View style={styles.bookInfoContainer}>
                        <View style={styles.bookTextRowContainer}>
                            <RobotoBoldText style={GLOBAL_STYLES.bookHeading} numberOfLines={1}>{this.props.title}</RobotoBoldText>
                        </View>
                        <View style={styles.bookTextRowContainer}>
                            <RobotoText style={GLOBAL_STYLES.bookInfo}>Autor:{' '}</RobotoText><RobotoBoldText style={GLOBAL_STYLES.bookInfo} numberOfLines={1}>{this.props.author}</RobotoBoldText>
                        </View>
                        <View style={styles.bookTextRowContainer}>
                            <RobotoText style={GLOBAL_STYLES.bookInfo}>Rok:{' '}</RobotoText>
                            <RobotoBoldText  style={GLOBAL_STYLES.bookInfo} numberOfLines={1}>{this.props.year}</RobotoBoldText>
                        </View>
                        <View style={styles.bookTextChildrenContainer}>
                            {this.props.children}
                        </View>
                    </View>
                </View>
            </TouchableHighlight>
        );
    }
}

// const imgNotAvailable = require('../assets/images/book_image_not_available.png');

const styles = StyleSheet.create({
    bookContainer: {
        flexDirection: "row",
        marginLeft: 10,
        marginRight: 10,
    },
    bookImage: {
        height: 145,
        width: 100
    },
    bookInfoContainer: {
        flexDirection: "column",
        marginLeft: 10,
    },
    bookTextRowContainer: {
        flexDirection: "row",
    },
    bookTextChildrenContainer: {
        flexDirection: "row",
        marginTop: 4
    },

});
