import React from "react";
import {ActivityIndicator, ScrollView, StyleSheet, View} from "react-native";
import {StyleConstants} from "style/StyleConstants";
import {RobotoBoldText, RobotoText} from "components/text/StyledText";
import {Button, Image} from "react-native-elements";
import {GLOBAL_STYLES} from "style/GlobalStyles";
import {useNavigation} from "@react-navigation/native";
import {StackNavigationProp} from "@react-navigation/stack";
import {CatalogScreens} from "navigation/enums/CatalogScreens";

//todo
type Props = {
    route: any;
}

export const BookDetailScreen: React.FC<Props> = (props: Props) => {

    const book = props.route.params.book;

    const navigation = useNavigation<StackNavigationProp<any>>();


    return (
            <ScrollView contentContainerStyle={styles.container}>
                <RobotoBoldText style={[GLOBAL_STYLES.bookHeading, {fontSize: 18, marginBottom: 10}]}>{book.title}</RobotoBoldText>
                <Image style={styles.bookImage}
                           source={{uri : `https://www.obalkyknih.cz/api/cover?isbn=${book.isbn}`}}
                           PlaceholderContent={<ActivityIndicator
                               style={GLOBAL_STYLES.indicator}/>
                           }
                    />


                <Button title={"Exempláře"} buttonStyle={{backgroundColor: StyleConstants.PRIMARY_BLUE_COLOR}} onPress={() => navigation.navigate(CatalogScreens.BOOK_ITEMS, {docNumber: book.docNumber})}/>

                <View style={styles.infoContainer}>
                    <RobotoBoldText style={styles.bookInfoHeading}>DocNumber</RobotoBoldText>
                    <RobotoText style={styles.bookInfo}>{book.docNumber}</RobotoText>

                    <RobotoBoldText style={styles.bookInfoHeading}>Autor</RobotoBoldText>
                    <RobotoText style={styles.bookInfo}>{book.author}</RobotoText>

                    <RobotoBoldText style={styles.bookInfoHeading}>Rok</RobotoBoldText>
                    <RobotoText style={styles.bookInfo}>{book.year}</RobotoText>

                    <RobotoBoldText style={styles.bookInfoHeading}>Počet stran</RobotoBoldText>
                    <RobotoText style={styles.bookInfo}>{book.pages}</RobotoText>

                    <RobotoBoldText style={styles.bookInfoHeading}>ISBN</RobotoBoldText>
                    <RobotoText style={styles.bookInfo}>{book.isbn}</RobotoText>

                    <RobotoBoldText style={styles.bookInfoHeading}>Verze</RobotoBoldText>
                    <RobotoText style={styles.bookInfo}>{book.version}</RobotoText>

                    <RobotoBoldText style={styles.bookInfoHeading}>Signatura</RobotoBoldText>
                    <RobotoText style={styles.bookInfo}>{book.signature}</RobotoText>
                </View>


            </ScrollView>
        );

    
};

const styles = StyleSheet.create({
    container: {
        marginTop: 10,
        paddingBottom: 20,
        alignItems: 'center',
        backgroundColor: StyleConstants.BACKGROUND,
    },
    infoContainer: {
        alignSelf: "flex-start",
        margin: 10
    },
    text: {
        color: StyleConstants.PRIMARY_BLUE_COLOR,
        fontSize: 16,
        justifyContent: "center",
        alignItems: "center",
        alignContent: "center",
    },
    bookInfoHeading: {
        ...GLOBAL_STYLES.bookInfo,
        // fontSize: 14,
    },
    bookInfo: {
        ...GLOBAL_STYLES.bookInfo,
        marginBottom: 8,
        // fontSize: 14,
    },
    bookImage: {
        height: 175,
        width: 130,
        marginBottom: 14,
    },

});