import React from "react";
import {Keyboard, Modal, StyleSheet, View,} from "react-native";

import AlephService from "services/AlephService";
import {Button, Input} from "react-native-elements";
import Constants from "expo-constants";
import {StyleConstants} from "../style/StyleConstants";
import User from "model/User";
import {GLOBAL_STYLES} from "../style/GlobalStyles";
import {AuthContext} from "../context/AuthContext";
import {RobotoBoldText, RobotoText} from "../components/text/StyledText";

export const LoginScreen : React.FC<{}> = () => {

    const { signIn } = React.useContext(AuthContext);

    const usernameInput = React.createRef<Input>();
    const passwordInput = React.createRef<Input>();

    const [username, setUsername] = React.useState('');
    const [password, setPassword] = React.useState('');
    const [modalVisible, setModalVisible] = React.useState(false);
    const [errorMessage, setErrorMessage] = React.useState('');

    const submit = async () => {
        try {
            const user: User = await AlephService.getUserInfo(username, password);
            user.password = password;
            signIn(user);
        } catch (e) {
            usernameInput.current.shake();
            passwordInput.current.shake();
            setErrorMessage(e.message);
            console.log(e);
        }
    };

    return (
            <View style={styles.container}>
                <RobotoBoldText h3 style={{marginBottom: 80, marginLeft: 8, marginTop: 0, color: StyleConstants.PRIMARY_BLUE_COLOR}}>Přihlaste se svým univerzitním účtem</RobotoBoldText>
                {errorMessage.length > 0 && <RobotoBoldText style={{color: 'red'}}>{errorMessage}</RobotoBoldText>}
                <Input
                    ref={usernameInput}
                    placeholder="Přihlašovací jméno"
                    onChangeText={setUsername}
                    placeholderTextColor={StyleConstants.PRIMARY_BLUE_COLOR}
                    inputStyle={GLOBAL_STYLES.input}
                    labelStyle={GLOBAL_STYLES.input}
                    returnKeyType = { "next" }
                    onSubmitEditing={() => { passwordInput.current.focus(); }}
                    blurOnSubmit={false}
                    textContentType="username"
                    autoCapitalize="none"
                />
                <Input
                    ref={passwordInput}
                    placeholder="Heslo"
                    onBlur={Keyboard.dismiss}
                    onChangeText={setPassword}
                    placeholderTextColor={StyleConstants.PRIMARY_BLUE_COLOR}
                    inputStyle={GLOBAL_STYLES.input}
                    labelStyle={GLOBAL_STYLES.input}
                    secureTextEntry={true}
                    textContentType="password"
                    autoCapitalize="none"
                />
                <Button
                    containerStyle={{marginTop: 20, alignItems: 'flex-end'}}
                    title={"Přihlásit"}
                    onPress={submit}
                />

                <Button
                    containerStyle={{marginTop: 20}}
                    title={"Jak se registrovat ?"}
                    onPress={() => setModalVisible(true)}
                />
                <Modal visible={modalVisible} transparent={false}>
                    <View style={styles.container}>
                        <View style={{margin: 8}}>
                            <RobotoText style={{marginTop: 4}}>Registrace do knihovny nebyla nikdy jednodušší.</RobotoText>
                            <RobotoText style={{marginTop: 4}}>1. Přinesete si studentskou / zaměstnaneckou kartu (ISIC / ALIVE / ITIC / ID karta).</RobotoText>
                            <RobotoText style={{marginTop: 4}}>2. Vytiskneme vám Evidenční list, který obsahuje vaše kontaktní informace.</RobotoText>
                            <RobotoText style={{marginTop: 4}}>3. Podepíšete Evidenční list, čímž souhlasíte s podmínkami služeb knihovny ČZU specifikovanými v Knihovním řádu.</RobotoText>
                            <Button
                                containerStyle={{marginTop: 20}}
                                title={"Zpět"}
                                onPress={() => setModalVisible(false)}
                            />
                        </View>
                    </View>
                </Modal>
            </View>
        );


};

const styles = StyleSheet.create({
    container: {
        paddingTop: Constants.statusBarHeight,
        flex: 1,
        flexDirection: "column",
        justifyContent: 'center',
        alignItems: 'center',
        backgroundColor: StyleConstants.BACKGROUND,
    },
    inputText: {
        // color: StyleConstants.PRIMARY_BLUE_COLOR,
        margin: 10
    },
    hintText: {
        color: StyleConstants.PRIMARY_BLUE_COLOR,
        fontSize: 16,
        justifyContent: "center",
        alignItems: "center",
        alignContent: "center",
        marginTop: 10
    },
    indicator: {
        flex: 1,
        flexDirection: 'column',
        alignItems: 'center',
        justifyContent: 'center',
        paddingVertical: 10
    },
    itemSeparator: {
        height: 15,
        width: '100%',
    }
});